/*
 * Copyright (C) 2020 Dan Leinir Turthra Jensen <admin@leinir.dk>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) version 3, or any
 * later version accepted by the membership of KDE e.V. (or its
 * successor approved by the membership of KDE e.V.), which shall
 * act as a proxy defined in Section 6 of version 3 of the license.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "entrywrapper.h"

namespace KNSCore {
    class EntryWrapper::Private {
    public:
        Private(const EntryInternal& entry
        )
            : entry(entry)
        {}
        EntryInternal entry;
    };
}

KNSCore::EntryWrapper::EntryWrapper(const KNSCore::EntryInternal& entry, QObject* parent)
    : QObject(parent)
    , d(new Private(entry))
{}

KNSCore::EntryWrapper::~EntryWrapper()
{
    delete d;
}

KNSCore::EntryInternal KNSCore::EntryWrapper::entry() const
{
    return d->entry;
}
