/*
    Copyright 2015 by Gregor Mi <codestruct@posteo.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef KMORETOOLSPRESETS_P_H
#define KMORETOOLSPRESETS_P_H

class KMoreToolsPresetsPrivate
{
public:
    /**
     * Same as KMoreToolsPresets::registerServicesByGroupingNames
     * but sets @p firstMoreSectionDesktopEntryName to the desktop entry name
     * of the first service that should be in the more section by default.
     * If there is no such service @p firstMoreSectionDesktopEntryName
     * will not be changed.
     */
    static QList<KMoreToolsService*> registerServicesByGroupingNames(
        QString* firstMoreSectionDesktopEntryName, KMoreTools* kmt, const QStringList& groupingNames);
};

#endif
